@@ -24,7 +24,7 @@ def dag(no_metadata_dir):
     dag = create_dag(
         no_metadata_dir,
         description="A dag created without metadata",
-        schedule_interval="0 0 * * *",
+        schedule="0 0 * * *",
         default_args={
             "owner": "gusty",
             "depends_on_past": False,
